"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Discussions = void 0;
const pineapple_1 = require("@snapshot-labs/pineapple");
const agent_1 = require("../agent");
const abi_json_1 = __importDefault(require("./abi.json"));
const AGENT_ADDRESS = '0x0000000000000000000000000000000000000002';
class Discussions extends agent_1.Agent {
    async addCategory({ name, about, parent }) {
        const { cid } = await (0, pineapple_1.pin)({ name, about });
        const metadataURI = `ipfs://${cid}`;
        return await this.client.send(AGENT_ADDRESS, abi_json_1.default, 'addCategory', [parent, metadataURI]);
    }
    async editCategory({ category, name, about }) {
        const { cid } = await (0, pineapple_1.pin)({ name, about });
        const metadataURI = `ipfs://${cid}`;
        return await this.client.send(AGENT_ADDRESS, abi_json_1.default, 'editCategory', [category, metadataURI]);
    }
    async removeCategory({ category }) {
        return await this.client.send(AGENT_ADDRESS, abi_json_1.default, 'removeCategory', [category]);
    }
    async addTopic({ title, content, author, category, parent }) {
        const { cid } = await (0, pineapple_1.pin)({ title, content });
        const metadataURI = `ipfs://${cid}`;
        return await this.client.send(AGENT_ADDRESS, abi_json_1.default, 'addTopic', [
            author,
            category,
            parent,
            metadataURI
        ]);
    }
    async editTopic({ topic, title, content }) {
        const { cid } = await (0, pineapple_1.pin)({ title, content });
        const metadataURI = `ipfs://${cid}`;
        return await this.client.send(AGENT_ADDRESS, abi_json_1.default, 'editTopic', [topic, metadataURI]);
    }
    async removeTopic({ topic }) {
        return await this.client.send(AGENT_ADDRESS, abi_json_1.default, 'removeTopic', [topic]);
    }
    async pinTopic({ topic }) {
        return await this.client.send(AGENT_ADDRESS, abi_json_1.default, 'pinTopic', [topic]);
    }
    async unpinTopic({ topic }) {
        return await this.client.send(AGENT_ADDRESS, abi_json_1.default, 'unpinTopic', [topic]);
    }
    async vote({ voter, topic, choice }) {
        return await this.client.send(AGENT_ADDRESS, abi_json_1.default, 'vote', [voter, topic, choice]);
    }
    async unvote({ voter, topic }) {
        return await this.client.send(AGENT_ADDRESS, abi_json_1.default, 'unvote', [voter, topic]);
    }
}
exports.Discussions = Discussions;
