"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Client = void 0;
const providers_1 = require("@ethersproject/providers");
const contracts_1 = require("@ethersproject/contracts");
const constants_1 = require("./constants");
const profiles_1 = require("./agents/profiles");
const discussions_1 = require("./agents/discussions");
const votes_1 = require("./agents/votes");
class Client {
    url;
    provider;
    signer;
    profiles;
    discussions;
    votes;
    constructor(options) {
        this.url = options?.url || constants_1.HIGHLIGHT_TESTNET;
        this.provider = options?.provider || new providers_1.StaticJsonRpcProvider(this.url, constants_1.CHAIN_ID);
        if (options?.signer)
            this.setSigner(options.signer);
        this.discussions = new discussions_1.Discussions(this);
        this.profiles = new profiles_1.Profiles(this);
        this.votes = new votes_1.Votes(this);
    }
    setSigner(signer) {
        this.signer = signer.connect(this.provider);
    }
    async getUnitReceipt(hash) {
        const init = {
            method: 'POST',
            headers: {
                Accept: 'application/json',
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                jsonrpc: '2.0',
                method: 'hl_getUnitReceipt',
                params: {
                    hash
                },
                id: null
            })
        };
        const res = await fetch(this.url, init);
        const { result } = await res.json();
        return result;
    }
    async send(address, abi, fn, args) {
        const contract = new contracts_1.Contract(address, abi, this.signer);
        return await contract[fn](...args);
    }
}
exports.Client = Client;
